(constant '*2pi* (mul 3.14159265358979323846264338327950288419716939937510 2))
(constant '*im1* '(0 1))

(define (re a) (nth 0 a))
(define (im a) (nth 1 a))
(define (fst a) (nth 0 a))
(define (snd a) (nth 1 a))

(define (cmplx a (b 0)) (list a b))
(define (conjz a)
	(if (number? a)
		(a)
		(list (im a) (- (re a)))
	)
)

(define (cmod a)
	(sqrt (add (pow (re a) 2) (pow (im a) 2)))
)

(define (cadd a b)
	(let 	((v1 (if (number? a)
			(cmplx a)
			a))
		(v2 (if (number? b)
			(cmplx b)
			b)))
	(list 	(add (re v1) (re v2))
		(add (im v1) (im v2))))
)

(define (csub a b)
	(let 	((v1 (if (number? a)
			(cmplx a)
			a))
		(v2 (if (number? b)
			(cmplx b)
			b)))
		(list 	(sub (re v1) (re v2))
			(sub (im v1) (im v2))))
)

(define (cmul a b)
	(let 	((v1 (if (number? a)
			(cmplx a)
			a))
		(v2 (if (number? b)
			(cmplx b)
			b)))
		(list 	(sub (mul (re v1) (re v2)) (mul (im v1) (im v2)))
			(add (mul (im v1) (re v2)) (mul (re v1) (im v2)))))
)

(define (cdiv a b)
	(let 	((v1 (if (number? a)
			(cmplx a)
			a))
		(v2 (if (number? b)
			(cmplx b)
			b)))
		(list 	(div (add (mul (re v1) (re v2)) (mul (im v1) (im v2))) 
							(pow (cmod b) 2))
			(div (sub (mul (im v1) (re v2)) (mul (re v1) (im v2))) 
							(pow (cmod b) 2))))
)

(define (csqrt a)
	(if (number? a)
		(sqrt a)
	(let 	((r1 (sqrt (div (add (re a) (cmod a)) 2)))
		(r2 (sqrt (div (sub (cmod a) (re a)) 2))))
		(list r1 (mul (sgn (im a)) r2)))
	)
)

(define (cpolar a)
	(let 	((r1 (cmod a))
  		(r2 (atan2 (im a) (re a))))
  	(list r1 r2))
)

(define (cexp a)
	(if (number? a)
		(exp a)
		(begin 
			(list 	(mul (exp (re a)) (cos (im a)))
				(mul (exp (re a)) (sin (im a)))
			))
	)
)

(define (cpow a n)
	(if (number? a)
		(pow a n)
	(let ((v1 (cpolar a)))
		(list 	(mul (pow (fst v1) n) (cos (mul n (snd v1))))
			(mul (pow (fst v1) n) (sin (mul n (snd v1))))))
	)
)

(define (cpowc a b)
	(let 	((co1 (mul (pow (fst (cpolar a)) (fst b)) 
				(exp (mul (sub (snd b)) (snd (cpolar a))))))
		(co2 (add (mul (fst b) (snd (cpolar a))) 
				(mul (snd b) (log (fst (cpolar a)))))))
		(list (mul co1 (cos co2))
					(mul co1 (sin co2)))
	)
)

(define (cln a k)
	(if (number? a)
		(log a)
	(let ((v1 (cpolar a))
				(mpi (mul *2pi* k)))
    (list (log (fst v1))
					(add mpi (snd v1))))
	)
)

(define (esqrt a)
	(if 
	(list? a) (csqrt a)
	(< a 0) (cmul (sqrt (- a)) *im1*)
	(>= a 0) (sqrt a))
)

(define (csqrtn a n k)
	(if 
	(number? a) (pow a (div 1 n))
	(or (< k 0) (>= k n)) (println "Wrong multiplicity factor!")
	(let 	((v1 (cpolar a))
		(mpi (mul *2pi* k)))
		(list
			(mul (pow (re v1) (div 1 n)) 	(cos (div n 
							(add mpi (im v1)))))
			(mul (pow (re v1) (div 1 n)) 	(sin (div n 
							(add mpi (im v1)))))))
	)
)

(define (csin a)
	(if (number? a)
		(sin a)
	(let 	((p1 (cmul *im1* a))
		(co1 (cmul 0.5 *im1*)))
		(cmul co1 (csub (cexp (cmul (- 1) p1)) (cexp p1))))
	)
)

(define (ccos a)
	(if (number? a)
		(cos a)
	(let 	((p1 (cmul *im1* a)))
		(cmul 0.5 (cadd (cexp (cmul (- 1) p1)) (cexp p1))))
	)
)

; eof